let nbChoiceLists = 2;
let nbChoiceListsTotal = 2;
let nbChoiceListsMax = 5;

let values = [];
let labels = [];
  
let preselectedUri = ["Q58637583", "Q18384454", "Q51711012", "Q18810063", "Q18810062"];

/*
 * Managing the creation of the menu (lists of kingdoms)
 */
function init()
{
  for(let i = 0; i < nbChoiceLists; i++)
  {
    addList(i);
  }

  let endPoint = "https://query.wikidata.org/sparql";
  
  let request = '';

  request += 'SELECT ?title ?titleLabel (COUNT(?monarch) AS ?monarchCount) ';
  request += 'WHERE ';
  request += '{ ';
  request += '  ?monarch wdt:P31 wd:Q5 ; ';
  request += '  p:P39 [ ps:P39 ?title ; pq:P580 ?startTime ; pq:P582 ?endTime ] . ';
  request += '  ?title wdt:P279* wd:Q116 . ';
  request += '  SERVICE wikibase:label { bd:serviceParam wikibase:language "[AUTO_LANGUAGE],en,fr" } ';
  request += '} ';
  request += 'GROUP BY ?title ?titleLabel ';
  request += 'ORDER BY DESC (?monarchCount)';
  
  let queryURL = endPoint + "?" + "query="+escape(request) + "&format=json";

  // launch the query
  let ajaxRequest = $.ajax({
    url:queryURL,
    dataType: 'json'
  });

  ajaxRequest.fail(function()
  {
    // if the request fails, return to menu
    console.log("EndPoint : " + endPoint);
    console.log("Requete : " + request);
    alert("Echec de la récupération des données");
  });

  ajaxRequest.done(function(data)
  {
    $(".choice_list").prepend('  <option value=""></option>');
    values.push("");
    labels.push("");
    
    // Add data to list
    for(let i = 0; i < data.results.bindings.length; i++)
    {
      let monarchCount = data.results.bindings[i]["monarchCount"]["value"];
      let titleUri = data.results.bindings[i]["title"]["value"];
      let titleLabel = data.results.bindings[i]["titleLabel"]["value"];
      
      let titleArray = titleUri.split("/");
      let value = titleArray[titleArray.length - 1];
      
      $(".choice_list").prepend('  <option value="'+value+'">'+titleLabel+' ('+monarchCount+')</option>');
      
      values.push(value);
      labels.push(titleLabel+' ('+monarchCount+')');
    }

    // Auto select
    $(".choice_list").each(function()
    {
        var htmlId = $(this).attr('id');
        var num = htmlId.split('_')[htmlId.split('_').length - 1];
        //$("#choice_list").val(preselectedUri[num-1]);
        $(this).val(preselectedUri[num]);
    });
    
    $('#loading').html('');
  });
  
  // Menage add choice list
  $("#add_choice_list").click(function() 
  {
    if(nbChoiceListsTotal < nbChoiceListsMax)
    {
      nbChoiceLists ++;
      nbChoiceListsTotal ++;
      
      addList(nbChoiceLists);
      
      if(nbChoiceListsTotal == nbChoiceListsMax)
      {
        $("#add_choice_list").css("display", "none");
      }
    }
  });
}

/*
 * Remove a list
 */
function removeList(e)
{
  nbChoiceListsTotal --;
  $("#add_choice_list").css("display", "inline-block");
  
  let idTab = $(e.target)[0].id.split("_");
  let id = idTab[idTab.length - 1];
    
  $("#div_choice_list"+id).remove();
}

/* 
 * Add a kindom list
 * @param {Number}              num             The num of the list
 */
function addList(num)
{
  var content = "<div id='div_choice_list"+num+"' class='div_choice_list'><select name='choice_list_"+num+"' id='choice_list_"+num+"' class='choice_list'></select>";
  if(num != 0)
  {
    content += "<img class='remove_choice_list' id='remove_choice_list_"+num+"' src='img/remove.png' /><br/></div>";
  }
  $("#choice_lists").append(content);
  
  // Add values to the list
  for(let i = 0; i < values.length; i++)
  {
    $("#choice_list_"+num).prepend('  <option value="'+values[i]+'">'+labels[i]+'</option>');
  }

  $("#choice_list_"+num).val(preselectedUri[num-1]);
  
  // Menage remove
  $("#remove_choice_list_"+num).click(function(e) 
  {
    removeList(e);
  });
}