﻿/**
 * TimeLine of the graph
 * @property {Vis.DataSet}                   items                           Dataset of timeline
 * @property {vis.Timeline}                  timeline                        The vis timeline
 * @property {RequestLauncher}               requestLauncher                 The request launcher
 * @property {FamilyGraph}                   familyGraph                     The graph of family
 * @property {MonarchInformation[]}          listMonarchTitleInfos           Array of monarch title informations
 * @property {DataTimeline[]}                data                            Array of all data display in the timeline
 * @property {Boolean[]}                     dataGet                         Array of data recovery state (True if Ok)
 */
TimeLineHistory = function()
{
  this.items = null;
  this.timeline = null;

  this.requestLauncher = new RequestLauncher();
  
  this.familyGraph = new FamilyGraph();

  this.monarchTitlesInfos = [];
  
  this.data = [];
  this.dataGet = [];
};

/**
 * Init the timeline 
 */
TimeLineHistory.prototype.init = function()
{
  $("#topInfos").html("Récupération des données");

  this.items = new vis.DataSet([]);
 
  let container = document.getElementById('timeLine');
    
  // Configuration of the Timeline
  let options = {};
  
  // Get ars values of address bar
  let args = location.search.substr(1).split(/&/);
  let argsValues = [];
  for(let i =0; i < args.length; i++)
  {
    let tmp = args[i].split(/=/);
    if (tmp[0] != "") {
        argsValues[decodeURIComponent(tmp[0])] = decodeURIComponent(tmp.slice(1).join("").replace("+", " "));
    }
  }
  
  let listMonarchTitlesUri = [];
  
  // Gets args values for init monarch list
  for (i in argsValues)
  {
    if(argsValues[i] != undefined && argsValues[i] != "")
    {
      listMonarchTitlesUri.push(argsValues[i]);
    }
  }

  // Get data and init timeLine
  let me = this;
  this.getAllDatas(listMonarchTitlesUri, function()
  {
    $("#topInfos").html("");

    let groups = me.initGroup();

    me.timeline = new vis.Timeline(container);
    me.timeline.setOptions(options);
    me.timeline.setGroups(groups);
    me.timeline.setItems(me.items);
    
    me.timeline.addCustomTime(new Date(), 'start');
    me.timeline.addCustomTime(new Date(), 'end');
    
    // If mouse over item : display item name in the topInfos div
    me.timeline.on('itemover', function (properties)
    {
      let items = me.items.get(properties.item);

      $("#topInfos").html(items.content);
    });
    
    // If mouse out item : empty the topInfos div
    me.timeline.on('itemout', function (properties)
    {
      $("#topInfos").html("");
    });

    // Menage click on an item
    me.timeline.on('click', function (properties) 
    {
      if(properties.item != null)
      {
        let item = me.items.get(properties.item);

        me.selectItem(item);
      }
    });
    
  });
};

/**
 * Menage request of all manarchTitle for get all datas, and call the callback if end
 * @param {String[]}             listMonarchTitle                List of monarch title URI
 * @param {Function}             callback                        The callback
 */
TimeLineHistory.prototype.getAllDatas = function(listMonarchTitle, callback)
{
  let me = this;
  
  for(let i = 0; i < listMonarchTitle.length; i++)
  {
    me.dataGet.push(false);
  }
  
  // 
  for(let i = 0; i < listMonarchTitle.length; i++)
  {
    me.monarchTitlesInfos.push(new MonarchInformation(listMonarchTitle[i]));

    me.getTitleInfos(listMonarchTitle[i], i, function()
    {
      me.getData(listMonarchTitle[i], (i+1), function(number)
      {
        me.dataGet[number] = true;
        
        let finish = true;
        for(let j = 0; j < me.dataGet.length; j++)
        {
          if(me.dataGet[j] == false)
            finish = false;
        }
        if(finish)
        {
          callback();
        }
      });
    });
  }
}

/**
 * Get the title informations (images) from wikidata
 * @param {String}               monarchTitle             The monarch title on which we're going to get data
 * @param {Number}               number                   The monarch title number
 * @param {Function}             callback                 The callback
 */
TimeLineHistory.prototype.getTitleInfos = function(monarchTitle, number, callback)
{
  var me = this;

  let request = "";
  
  request += "SELECT ?titleLabel ?image ?flag ?coatOfArms ";
  request += "WHERE ";
  request += "{ ";
  request += "wd:" + monarchTitle + " rdfs:label ?titleLabel . ";
  request += "OPTIONAL { wd:" + monarchTitle + " wdt:P18 ?image } ";
  request += "OPTIONAL { wd:" + monarchTitle + " wdt:P41 ?flag } ";
  request += "OPTIONAL { wd:" + monarchTitle + " wdt:P94 ?coatOfArms } ";
  request += "FILTER(LANG(?titleLabel) = 'en') ";
  request += "} ";

  me.requestLauncher.launch(request, function(resultDatas)
  {
    if(resultDatas.results.bindings.length > 0)
    {
      let data = resultDatas.results.bindings[0];

      if(data["titleLabel"] != null && data["titleLabel"] != "") 
      {
        me.monarchTitlesInfos[number].label = data["titleLabel"]["value"];
      }

      if(data["flag"] != null && data["flag"] != "") 
      {
        me.monarchTitlesInfos[number].image = data["flag"]["value"];
      }
      else if(data["coatOfArms"] != null && data["coatOfArms"] != "") 
      {
        me.monarchTitlesInfos[number].image = data["coatOfArms"]["value"];
      }
      else if(data["image"] != null && data["image"] != "") 
      {
        me.monarchTitlesInfos[number].image = data["image"]["value"];
      }

      callback();
    }
  });
};

/**
 * Launch an request 
 * @param {String}               monarchTitle             The monarch title on which we're going to get data
 * @param {String}               groupNumber              The number of the data in the function getAllDatas
 * @param {Function}             callback                 The callback with the groupNumber in param
 */
TimeLineHistory.prototype.getData = function(monarchTitle, groupNumber, callback)
{
  let me = this;
  
  // Initialize request
  let request = '';
  
  request += 'SELECT ?king ?kingLabel ?gender ?startTime ?endTime ';
  request += 'WHERE ';
  request += '{ ';
  request += '?king wdt:P31 wd:Q5 ; ';
  request += 'p:P39 [ ps:P39 wd:'+monarchTitle+' ; pq:P580 ?startTime ; pq:P582 ?endTime ] . ';
  request += ' OPTIONAL { ?king wdt:P21 ?gender } . '
  request += 'SERVICE wikibase:label { bd:serviceParam wikibase:language "[AUTO_LANGUAGE],fr,en" } ';
  request += '} ';
  request += 'ORDER BY (?startTime) ';
  
  me.requestLauncher.launch(request, function(resultDatas)
  {
    for(let i = 0; i < resultDatas.results.bindings.length; i++)
    {
      let kingUri = resultDatas.results.bindings[i]["king"]["value"];
      let kingLabel = resultDatas.results.bindings[i]["kingLabel"]["value"];
      let startTime = resultDatas.results.bindings[i]["startTime"]["value"];
      let endTime = resultDatas.results.bindings[i]["endTime"]["value"];
      let gender = resultDatas.results.bindings[i]["gender"]["value"];
      
      let dateStartTime = me.convertDate(startTime);
      let dateEndTime = me.convertDate(endTime);
      
      if(!me.itemAlreadyExist(kingUri, startTime, endTime))
      {
        // Init image for gender
        let imageContent = "";
        if(gender == "http://www.wikidata.org/entity/Q6581097")
        {
          imageContent += "<img src='img/king.png' style='width:16px; height:16px;vertical-align:middle;' />";
        }
        else if(gender == "http://www.wikidata.org/entity/Q6581072")
        {
          imageContent += "<img src='img/queen.png' style='width:16px; height:16px;vertical-align:middle;' />";
        }
          
        let content = '<div>'+kingLabel+' ' + imageContent + '</div>'
        
        // Add item dateTime
        me.items.add(
          [
            {
              id: kingUri+'_'+dateStartTime+'_'+dateEndTime, 
              uri:kingUri, 
              content: content, 
              start: dateStartTime, 
              end: dateEndTime, 
              startTime: startTime, 
              endTime: endTime, 
              group: monarchTitle,
              className: 'group' + groupNumber,
              type: 'range'
            }
          ]
        );
        
        // Add Data
        if(me.data[kingUri] == undefined)
        {
          me.data[kingUri] = new DataTimeLine(kingUri);
        }
        me.data[kingUri].addDate(startTime.split('T')[0], endTime.split('T')[0]);
      }
    }
    
    callback(groupNumber-1);
  });
};

/**
 * Initialize timeline group display
 * @return {vis.Dataset[]}             The created groups 
 */
TimeLineHistory.prototype.initGroup = function()
{
  var me = this;
  // Init groups
  let groups = new vis.DataSet([]);
  for(let i = 0; i < me.monarchTitlesInfos.length; i++)
  {
    let color = "";
    if(i == 0)
    {
      color = "LightGreen";
    }
    else if(i == 1)
    {
      color = "orange";
    }
    else if(i == 2)
    {
      color = "#82b3ff";
    }
    else if(i == 3)
    {
      color = "yellow";
    }
    else if(i == 4)
    {
      color = "#ff93ee";
    }

    let content = "";

    if(me.monarchTitlesInfos[i].image)
    {
      content = "<img src='"+me.monarchTitlesInfos[i].image+"' width='120' height='120' title='" + me.monarchTitlesInfos[i].label + "'' />";
    }
    else
    {
      content = "<p>"+me.monarchTitlesInfos[i].label+"</p>";
    }

    groups.add({id: me.monarchTitlesInfos[i].uri, content: content, style:"background-color : " + color + ";"});
  }
  
  return groups;
};



/**
 * Check if a item already exist in the item list
 * @param {String}          uri             URI of the item
 * @param {String}          startTime       Date of start
 * @param {String}          endTime         Date of end
 */
TimeLineHistory.prototype.itemAlreadyExist = function(uri, startTime, endTime)
{
  let itemList = this.items.get({
    filter: function(item) {
      return item.uri == uri && item.startTime == startTime && item.endTime == endTime;
    }
  });

  if(itemList.length > 0)
  {
    return true;
  }
  else
  {
    return false;
  }
};

/**
 * Method of selection of an iteam, get information in triplestore and display data and graph
 * @param {String}          item             URI of the item
 */
TimeLineHistory.prototype.selectItem = function(item)
{
  let me = this;
  
  // update cursor position
  me.timeline.setCustomTime(item.start, 'start');
  me.timeline.setCustomTime(item.end, 'end');
  
  // Update base information of the content
  let infosContent = "";
  infosContent += "<h3>"+item.content+"</h3>"
  infosContent += "<p><a href='"+item.uri+"'>"+item.uri+"</a></p>"
  infosContent += "<div id='infos_bis'><h4>Loading Informations ...</h4></div>";
  $('#infos').html(infosContent);
  
  // Update graph
  this.familyGraph.init(item.uri, item.content, me.items);
  
  // Get data of the person in the wikidata triplestore
  let request = '';
  request += "SELECT (GROUP_CONCAT(DISTINCT ?picture ; separator='_') as ?pictures) ?place_of_birth ?place_of_birthLabel ?place_of_death ?place_of_deathLabel ?gender ?genderLabel ?date_of_birth ?date_of_death ?manner_of_death ?manner_of_deathLabel ?cause_of_death ?cause_of_deathLabel ?family ?familyLabel ";
  request += 'WHERE ';
  request += '{ ';
  request += '  OPTIONAL { <'+item.uri+'> wdt:P18 ?picture } . ';
  request += '  OPTIONAL { <'+item.uri+'> wdt:P19 ?place_of_birth } . ';
  request += '  OPTIONAL { <'+item.uri+'> wdt:P20 ?place_of_death } . ';
  request += '  OPTIONAL { <'+item.uri+'> wdt:P21 ?gender } . ';
  request += '  OPTIONAL { <'+item.uri+'> wdt:P569 ?date_of_birth } . ';
  request += '  OPTIONAL { <'+item.uri+'> wdt:P570 ?date_of_death } . ';
  request += '  OPTIONAL { <'+item.uri+'> wdt:P1196 ?manner_of_death } . ';
  request += '  OPTIONAL { <'+item.uri+'> wdt:P509 ?cause_of_death } . ';
  request += '  OPTIONAL { <'+item.uri+'> wdt:P53 ?family } ';
  request += '  SERVICE wikibase:label { bd:serviceParam wikibase:language "[AUTO_LANGUAGE],fr,en" } ';
  request += '} ';
  request += 'GROUP BY ?place_of_birth ?place_of_birthLabel ?place_of_death ?place_of_deathLabel ?gender ?genderLabel ?date_of_birth ?date_of_death ?manner_of_death ?manner_of_deathLabel ?cause_of_death ?cause_of_deathLabel ?family ?familyLabel';
  
  me.requestLauncher.launch(request, function(resultDatas)
  {
    if(resultDatas != null)
    {
      if(resultDatas.results.bindings.length > 0)
      {
        let data = resultDatas.results.bindings[0];
        
        let pictures = "";
        let placeOfBirth = "";
        let placeOfDeath = "";
        let gender = "";
        let dateOfBirth = "";
        let dateOfDeath = "";
        let mannerOfDeath = "";
        let causeOfDeath = "";
        let family = "";
        
        if(data["pictures"] != null) pictures = data["pictures"]["value"];
        if(data["genderLabel"] != null) gender = data["genderLabel"]["value"];
        if(data["date_of_birth"] != null) dateOfBirth = data["date_of_birth"]["value"];
        if(data["date_of_death"] != null) dateOfDeath = data["date_of_death"]["value"];
        if(data["manner_of_deathLabel"] != null) mannerOfDeath = data["manner_of_deathLabel"]["value"];
        if(data["cause_of_deathLabel"] != null) causeOfDeath = data["cause_of_deathLabel"]["value"];
        if(data["familyLabel"] != null) family = data["familyLabel"]["value"];
        if(data["place_of_birthLabel"] != null && data["place_of_birthLabel"] != "") 
        {
          placeOfBirth = data["place_of_birthLabel"]["value"];
        }
        else
        {
          placeOfBirth = "(unknown)";
        }
        if(data["place_of_deathLabel"] != null && data["place_of_deathLabel"] != "") 
        {
          placeOfDeath = data["place_of_deathLabel"]["value"];
        }
        else
        {
          placeOfDeath = "(unknown)";
        }
        
        let content = "";
        
        // Add images
        if(pictures != null && pictures != "")
        {
          content += "<p>Pictures : </p>";
          
          for(let i = 0; i < pictures.split('_').length; i++)
          {
            content += '<img class="image_infos" src="'+pictures.split('_')[i]+'" />'
          }
        }
       
        // Updating the display from the recovered data
        content += "<p>Gender : " + gender + "</p>";
        content += "<p>Family : " + family + "</p>";
        
        let lifeDuration = new Date(me.convertDate(dateOfDeath).getTime() - me.convertDate(dateOfBirth).getTime());
        content += "<p>Born in " + placeOfBirth + " on  " + dateOfBirth.split("T")[0] + "</p>";
        content += "<p>Death in " + placeOfDeath + " on  " + dateOfDeath.split("T")[0] + " at " + (lifeDuration.getUTCFullYear() - 1970)  + " years</p>";
        
        if(mannerOfDeath != null && mannerOfDeath != "")
        {
          content += "<p>Manner of death : " + mannerOfDeath + "</p>";
        }
        if(causeOfDeath != null && causeOfDeath != "")
        {
          content += "<p>Cause of death : " + causeOfDeath + "</p>";
        }
        
        // Draw reigns informations
        if(me.data[item.uri] != undefined)
        {
          content += "<h4>Reigns : </h4>";
          
          for(let i = 0; i < me.data[item.uri].startDate.length; i++)
          {
            let start = me.data[item.uri].startDate[i];
            let end = me.data[item.uri].endDate[i];
            
            let diff = new Date(me.convertDate(end).getTime() - me.convertDate(start).getTime());

            content += "<p>" + start + " / " + end + " (" + (diff.getUTCFullYear() - 1970) + " years)</p>";
          }
        }
        
        $('#infos_bis').html(content);
      }
      else
      {
        $('#infos_bis').html("");
      }
    }
  });
  
};

/**
 * Convert a Wikidata date to a js date
 * @param {String}          dateStr             Wikidata date as a string
 */
TimeLineHistory.prototype.convertDate = function(dateStr)
{
  let date = new Date();
  
  dateStr = dateStr.split("T")[0];
  
  if(dateStr.split("-")[0] == "")
  { 
    date.setFullYear('-' + dateStr.split("-")[1]);
    date.setMonth(dateStr.split("-")[2]);
    date.setDate(dateStr.split("-")[3]);
  }
  else
  {
    date.setFullYear(dateStr.split("-")[0]);
    date.setMonth(dateStr.split("-")[1]);
    date.setDate(dateStr.split("-")[2]);
  }
  
  return date; 
};
